==> 0.0.78.tar.gz: lib/src/widgets/data_factory/form_factory/phone_number_list_form.dart <==
import 'package:collection/collection.dart';
import 'package:verttice_logic/verttice_logic.dart';
import "package:flutter/material.dart";
import "package:flutter_multi_formatter/flutter_multi_formatter.dart";
import "package:url_launcher/url_launcher.dart";
import 'package:verttice_ui/src/widgets/data_factory/common/common_index.dart';
import 'package:verttice_ui/verttice_ui.dart';

class PhoneNumberListFormFactory extends StatefulWidget {
  final String displayName;
  final String name;
  final Function(String name, dynamic value) onChanged;
  final List<Map<String, dynamic>>? value;
  final FormFieldState<dynamic>? state;
  const PhoneNumberListFormFactory({
    Key? key,
    this.value,
    required this.displayName,
    required this.name,
    required this.onChanged,
    this.state,
  }) : super(key: key);

  @override
  _TextListState createState() => _TextListState();
}

class _TextListState extends State<PhoneNumberListFormFactory> {
  List<Map<String, dynamic>> _value = [];
  List<PhoneNumberModel> _newdData = [];
  late StateSetter _setState;

  @override
  void initState() {
    super.initState();
    _value = widget.value ?? [];
    _newdData =
        (widget.value ?? []).map((e) => PhoneNumberModel.fromMap(e)).toList();
  }

  @override
  void didUpdateWidget(covariant PhoneNumberListFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (!const DeepCollectionEquality().equals(_value, widget.value)) {
      _value = widget.value ?? [];
      _newdData =
          (widget.value ?? []).map((e) => PhoneNumberModel.fromMap(e)).toList();
    }
  }

  @override
  Widget build(BuildContext context) {
    return BorderLabel(
      label: widget.displayName,
      child: SelectableText(
        visualPhoneNumberList(_value, context: context).value,
      ),
      onPressed: () async {
        await showDialog(
          context: context,
          useSafeArea: true,
          builder: (BuildContext context) {
            return StatefulBuilder(builder: (BuildContext context, setState) {
              _setState = setState;

              return SimpleDialog(
                insetPadding: const EdgeInsets.all(6),
                contentPadding: const EdgeInsets.all(6),
                children: [
                  SizedBox(
                    height: 400,
                    width: 400,
                    child: Scaffold(
                      appBar: ThemedAppBar(
                        title: widget.displayName,
                        actions: [
                          _AddTextFieldListItem(
                            onChanged: (PhoneNumberModel item) {
                              _setState(() {
                                _newdData.add(item);
                              });
                            },
                          ),
                          IconButton(
                            icon: ThemedIcons.check(),
                            onPressed: () {
                              final phoneNumbers =
                                  _newdData.map((e) => e.toMap()).toList();

                              widget.onChanged(
                                widget.name,
                                phoneNumbers,
                              );
                              widget.state?.didChange(
                                phoneNumbers,
                              );
                              Navigator.of(context).pop();
                            },
                          ),
                        ],
                      ),
                      body: ListView.builder(
                        controller: ScrollController(),
                        itemCount: _newdData.length,
                        physics: const BouncingScrollPhysics(),
                        itemBuilder: (context, i) => _item(_newdData[i], i),
                      ),
                    ),
                  )
                ],
              );
            });
          },
        );

        setState(() {});
      },
    );
  }

  Widget _item(PhoneNumberModel item, int i) {
    return Row(
      children: [
        Expanded(
          child: _TextFieldListItem(
            value: item,
            onChanged: (val) {
              _setState(() {
                _newdData = _newdData
                    .asMap()
                    .map((key, value) => MapEntry(key, key == i ? val : value))
                    .values
                    .map<PhoneNumberModel>((e) => e)
                    .toList();
              });
            },
          ),
        ),
        InkWell(
          onTap: () {
            _setState(() {
              _newdData.removeWhere((element) {
                return element == item;
              });
            });
          },
          child: CircleAvatar(
            backgroundColor: Colors.red,
            child: ThemedIcons.clear(
              iconStyle: IconStyle(
                color: Colors.white,
              ),
            ),
          ),
        )
      ],
    );
  }
}

class _TextFieldListItem extends StatefulWidget {
  final PhoneNumberModel value;
  final Function(PhoneNumberModel name) onChanged;

  const _TextFieldListItem({
    Key? key,
    required this.value,
    required this.onChanged,
  }) : super(key: key);

  @override
  _TextFieldListItemState createState() => _TextFieldListItemState();
}

class _TextFieldListItemState extends State<_TextFieldListItem> {
  PhoneNumberModel _value = PhoneNumberModel();
  PhoneNumberModel _newData = PhoneNumberModel();

  @override
  void initState() {
    super.initState();
    _value = widget.value;
    _newData = _value;
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant _TextFieldListItem oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_value != widget.value) {
      _value = widget.value;
      _newData = _value;
    }
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        sizedBoxH16,
        Row(
          children: [
            sizedBoxH8,
            Row(
              children: [
                Tooltip(
                  message: "Call",
                  child: IconButton(
                      icon: ThemedIcons.smartphone(),
                      onPressed: () async {
                        launch("tel://${widget.value}");
                      }),
                ),
                sizedBoxH8,
              ],
            ),
            Tooltip(
              message: "Edit",
              child: IconButton(
                icon: ThemedIcons.edit(),
                onPressed: () async {
                  await showDialog(
                    context: context,
                    builder: (BuildContext context) {
                      return StatefulBuilder(
                        builder: (BuildContext context, setState) {
                          return AlertDialog(
                            actions: [
                              TextButton(
                                onPressed: () {
                                  setState(() {
                                    _newData = _value;
                                  });

                                  Navigator.of(context).pop();
                                },
                                child: const Text("Cancel"),
                              ),
                              TextButton(
                                onPressed: () {
                                  if (!isPhoneValid(
                                      _newData.completeNumber ?? '')) {
                                    ThemedToast.error(
                                      context: context,
                                      message:
                                          "Please enter a valid phone number",
                                    );
                                  } else {
                                    widget.onChanged(_newData);

                                    setState(() {
                                      _value = _newData;
                                    });
                                    Navigator.of(context).pop();
                                  }
                                },
                                child: const Text("Save"),
                              ),
                            ],
                            content: Builder(
                              builder: (context) {
                                return SizedBox(
                                  height: 150,
                                  width: 400,
                                  child: Scaffold(
                                    body: Builder(
                                      builder: (context) {
                                        return PhoneNumberFormFactory(
                                          displayName: "Phone Number",
                                          name: "phoneNumber",
                                          onChanged: (name, value) {
                                            setState(() {
                                              _newData = value;
                                            });
                                          },
                                          value: _newData.toMap(),
                                        );
                                      },
                                    ),
                                  ),
                                );
                              },
                            ),
                          );
                        },
                      );
                    },
                  );
                },
              ),
            ),
            sizedBoxH8,
            SelectableText(
              widget.value.completeNumberWith0 ?? '',
              style: const TextStyle(
                overflow: TextOverflow.fade,
              ),
            ),
          ],
        ),
        sizedBoxH16,
        const Divider()
      ],
    );
  }
}

class _AddTextFieldListItem extends StatefulWidget {
  final Function(PhoneNumberModel name) onChanged;

  const _AddTextFieldListItem({
    Key? key,
    required this.onChanged,
  }) : super(key: key);

  @override
  _AddTextFieldListItemState createState() => _AddTextFieldListItemState();
}

class _AddTextFieldListItemState extends State<_AddTextFieldListItem> {
  PhoneNumberModel _newLine = PhoneNumberModel();

  @override
  void initState() {
    super.initState();
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant _AddTextFieldListItem oldWidget) {
    super.didUpdateWidget(oldWidget);
  }

  void _cleanState() {
    _newLine = PhoneNumberModel();
  }

  @override
  Widget build(BuildContext context) {
    return IconButton(
      onPressed: () async {
        await showDialog(
          context: context,
          builder: (BuildContext context) {
            return StatefulBuilder(
              builder: (BuildContext context, setState) {
                return AlertDialog(
                  actions: [
                    TextButton(
                      child: const Text("Cancel"),
                      onPressed: () {
                        _cleanState();
                        setState(() {});
                        Navigator.of(context).pop();
                      },
                    ),
                    TextButton(
                      child: const Text("Save"),
                      onPressed: () {
                        if (!isPhoneValid(_newLine.completeNumber ?? '')) {
                          ThemedToast.error(
                            context: context,
                            message: "Please enter a valid phone number",
                          );
                        } else {
                          widget.onChanged(_newLine);
                          setState(() {
                            _newLine = PhoneNumberModel();
                          });
                          Navigator.of(context).pop();
                        }

                        return;
                      },
                    ),
                  ],
                  content: Builder(
                    builder: (context) {
                      return SizedBox(
                        height: 150,
                        width: 400,
                        child: Scaffold(
                          body: Builder(
                            builder: (BuildContext context) {
                              return PhoneNumberFormFactory(
                                displayName: "Phone Number",
                                name: "phoneNumber",
                                onChanged: (name, value) {
                                  setState(() {
                                    _newLine = value;
                                  });
                                },
                                value: _newLine.toMap(),
                              );
                            },
                          ),
                        ),
                      );
                    },
                  ),
                );
              },
            );
          },
        );
      },
      icon: ThemedIcons.add(),
    );
  }
}

==> 0.0.80.tar.gz: lib/src/widgets/data_factory/form_factory/phone_number_list_form.dart <==
import 'package:collection/collection.dart';
import 'package:verttice_logic/verttice_logic.dart';
import "package:flutter/material.dart";
import "package:flutter_multi_formatter/flutter_multi_formatter.dart";
import "package:url_launcher/url_launcher.dart";
import 'package:verttice_ui/verttice_ui.dart';

class PhoneNumberListFormFactory extends StatefulWidget {
  final String displayName;
  final String name;
  final Function(String name, dynamic value) onChanged;
  final List<Map<String, dynamic>>? value;
  final FormFieldState<dynamic>? state;
  const PhoneNumberListFormFactory({
    Key? key,
    this.value,
    required this.displayName,
    required this.name,
    required this.onChanged,
    this.state,
  }) : super(key: key);

  @override
  _TextListState createState() => _TextListState();
}

class _TextListState extends State<PhoneNumberListFormFactory> {
  List<Map<String, dynamic>> _value = [];
  List<PhoneNumberModel> _newdData = [];
  late StateSetter _setState;

  @override
  void initState() {
    super.initState();
    _value = widget.value ?? [];
    _newdData = (widget.value ?? [])
        .map(
          (e) => PhoneNumberModel.fromMap(e),
        )
        .toList();
  }

  @override
  void didUpdateWidget(covariant PhoneNumberListFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (!const DeepCollectionEquality().equals(_value, widget.value)) {
      _value = widget.value ?? [];
      _newdData = (widget.value ?? [])
          .map(
            (e) => PhoneNumberModel.fromMap(e),
          )
          .toList();
    }
  }

  @override
  Widget build(BuildContext context) {
    return BorderLabel(
      label: widget.displayName,
      child: SelectableText(
        visualPhoneNumberList(_value, context: context).value,
      ),
      onPressed: () async {
        await showDialog(
          context: context,
          useSafeArea: true,
          builder: (BuildContext context) {
            return StatefulBuilder(builder: (BuildContext context, setState) {
              _setState = setState;

              return SimpleDialog(
                insetPadding: const EdgeInsets.all(6),
                contentPadding: const EdgeInsets.all(6),
                children: [
                  SizedBox(
                    height: 400,
                    width: 400,
                    child: Scaffold(
                      appBar: ThemedAppBar(
                        title: widget.displayName,
                        actions: [
                          _AddTextFieldListItem(
                            onAddSubmit: (PhoneNumberModel item) {
                              _setState(() {
                                _newdData.add(item);
                              });
                            },
                          ),
                          IconButton(
                            icon: ThemedIcons.check(),
                            onPressed: () {
                              final phoneNumbers =
                                  _newdData.map((e) => e.toMap()).toList();

                              widget.onChanged(
                                widget.name,
                                phoneNumbers,
                              );
                              widget.state?.didChange(
                                phoneNumbers,
                              );
                              Navigator.of(context).pop();
                            },
                          ),
                        ],
                      ),
                      body: ListView.builder(
                        controller: ScrollController(),
                        itemCount: _newdData.length,
                        physics: const BouncingScrollPhysics(),
                        itemBuilder: (context, i) => _item(_newdData[i], i),
                      ),
                    ),
                  )
                ],
              );
            });
          },
        );

        setState(() {});
      },
    );
  }

  Widget _item(PhoneNumberModel item, int i) {
    return Row(
      children: [
        Expanded(
          child: _TextFieldListItem(
            value: item,
            onEditSubmit: (val) {
              _setState(() {
                _newdData = _newdData
                    .asMap()
                    .map((key, value) => MapEntry(key, key == i ? val : value))
                    .values
                    .map<PhoneNumberModel>((e) => e)
                    .toList();
              });
            },
          ),
        ),
        InkWell(
          onTap: () {
            _setState(() {
              _newdData.removeWhere((element) {
                return element == item;
              });
            });
          },
          child: CircleAvatar(
            backgroundColor: Colors.red,
            child: ThemedIcons.clear(
              iconStyle: IconStyle(
                color: Colors.white,
              ),
            ),
          ),
        )
      ],
    );
  }
}

class _TextFieldListItem extends StatefulWidget {
  final PhoneNumberModel value;
  final Function(PhoneNumberModel name) onEditSubmit;

  const _TextFieldListItem({
    Key? key,
    required this.value,
    required this.onEditSubmit,
  }) : super(key: key);

  @override
  _TextFieldListItemState createState() => _TextFieldListItemState();
}

class _TextFieldListItemState extends State<_TextFieldListItem> {
  PhoneNumberModel _value = PhoneNumberModel();
  PhoneNumberModel _newData = PhoneNumberModel();
  late StateSetter _setState;

  @override
  void initState() {
    super.initState();
    _value = widget.value;
    _newData = _value;
  }

  @override
  void didUpdateWidget(covariant _TextFieldListItem oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_value != widget.value) {
      _value = widget.value;
      _newData = _value;
    }
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        sizedBoxH16,
        Row(
          children: [
            sizedBoxH8,
            Row(
              children: [
                Tooltip(
                  message: "Call",
                  child: IconButton(
                      icon: ThemedIcons.smartphone(),
                      onPressed: () async {
                        launch("tel://${widget.value}");
                      }),
                ),
                sizedBoxH8,
              ],
            ),
            Tooltip(
              message: "Edit",
              child: IconButton(
                icon: ThemedIcons.edit(),
                onPressed: () async {
                  await showDialog(
                    context: context,
                    builder: (BuildContext context) {
                      return StatefulBuilder(
                        builder: (BuildContext context, setState) {
                          _setState = setState;
                          return AlertDialog(
                            actions: [
                              TextButton(
                                onPressed: () {
                                  _setState(() {
                                    _newData = _value;
                                  });

                                  Navigator.of(context).pop();
                                },
                                child: const Text("Cancel"),
                              ),
                              TextButton(
                                onPressed: () {
                                  if (!isPhoneValid(
                                      _newData.completeNumber ?? '')) {
                                    ThemedToast.error(
                                      context: context,
                                      message:
                                          "Please enter a valid phone number",
                                    );
                                  } else {
                                    _setState(() {
                                      _value = _newData;
                                    });
                                    widget.onEditSubmit(_newData);

                                    Navigator.of(context).pop();
                                  }
                                },
                                child: const Text("Save"),
                              ),
                            ],
                            content: Builder(
                              builder: (context) {
                                return SizedBox(
                                  height: 150,
                                  width: 400,
                                  child: Scaffold(
                                    body: Builder(
                                      builder: (context) {
                                        return PhoneNumberFormFactory(
                                          displayName: "Phone Number",
                                          name: "phoneNumber",
                                          onChanged: (name, value) {
                                            _setState(() {
                                              _newData =
                                                  PhoneNumberModel.fromMap(
                                                DataClassUtils
                                                    .dynamicToMapStringDynamic(
                                                  value,
                                                ),
                                              );
                                            });
                                          },
                                          value: _newData.toMap(),
                                        );
                                      },
                                    ),
                                  ),
                                );
                              },
                            ),
                          );
                        },
                      );
                    },
                  );
                },
              ),
            ),
            sizedBoxH8,
            SelectableText(
              visualPhoneNumber(_value.toMap(), context: context).value,
              style: const TextStyle(
                overflow: TextOverflow.fade,
              ),
            ),
          ],
        ),
        sizedBoxH16,
        const Divider()
      ],
    );
  }
}

class _AddTextFieldListItem extends StatelessWidget {
  final Function(PhoneNumberModel name) onAddSubmit;

  const _AddTextFieldListItem({
    Key? key,
    required this.onAddSubmit,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return IconButton(
      onPressed: () async {
        await showDialog(
          context: context,
          builder: (BuildContext context) {
            return StatefulBuilder(
              builder: (BuildContext context, setState) {
                return _AddTextFieldListItemDialog(
                  onAddSubmit: onAddSubmit,
                );
              },
            );
          },
        );
      },
      icon: ThemedIcons.add(),
    );
  }
}

class _AddTextFieldListItemDialog extends StatefulWidget {
  final Function(PhoneNumberModel name) onAddSubmit;

  const _AddTextFieldListItemDialog({
    Key? key,
    required this.onAddSubmit,
  }) : super(key: key);

  @override
  _AddTextFieldListItemDialogState createState() =>
      _AddTextFieldListItemDialogState();
}

class _AddTextFieldListItemDialogState
    extends State<_AddTextFieldListItemDialog> {
  PhoneNumberModel _newLine = PhoneNumberModel();

  void _cleanState() {
    _newLine = PhoneNumberModel();
  }

  @override
  Widget build(BuildContext context) {
    return AlertDialog(
      actions: [
        TextButton(
          child: const Text("Cancel"),
          onPressed: () {
            _cleanState();
            setState(() {});
            Navigator.of(context).pop();
          },
        ),
        TextButton(
          child: const Text("Save"),
          onPressed: () {
            if (!isPhoneValid(_newLine.completeNumber ?? '')) {
              ThemedToast.error(
                context: context,
                message: "Please enter a valid phone number",
              );
            } else {
              widget.onAddSubmit(_newLine);
              setState(() {
                _newLine = PhoneNumberModel();
              });
              Navigator.of(context).pop();
            }

            return;
          },
        ),
      ],
      content: Builder(
        builder: (context) {
          return SizedBox(
            height: 150,
            width: 400,
            child: Scaffold(
              body: Builder(
                builder: (BuildContext context) {
                  return PhoneNumberFormFactory(
                    displayName: "Phone Number",
                    name: "phoneNumber",
                    onChanged: (name, value) {
                      setState(() {
                        _newLine = PhoneNumberModel.fromMap(
                          DataClassUtils.dynamicToMapStringDynamic(
                            value,
                          ),
                        );
                      });
                    },
                    value: _newLine.toMap(),
                  );
                },
              ),
            ),
          );
        },
      ),
    );
  }
}
